#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#define DIM_H 800
#define DIM_V 600
#define BUTTON_SX 1
#define BUTTON_DX 3

//strutture usate
typedef struct point
{
  int x;
  int y;
} point;

typedef struct RGB
{
  Uint8 R;
  Uint8 G;
  Uint8 B;
} RGB_Color;
  
//function grafiche
void draw_pixel(SDL_Surface *surface, int x, int y, RGB_Color s_color);
void draw_cross(SDL_Surface *surface, point p, RGB_Color c_color);
void draw_line(SDL_Surface *surface, point p0, point p1, RGB_Color color);
void draw_square(SDL_Surface *surface, point p, int lato, RGB_Color color);

int main()
{
  //superfici utilizzate
  SDL_Surface *screen;
  SDL_Surface *screen_backup;
  SDL_Surface *cursor;
  SDL_Surface *bg_cursor;
  SDL_Surface *temp;

  // si inizializza il sistema video
  if( SDL_Init(SDL_INIT_VIDEO) <0 )
  {
    // SDL_GetError() ritorna una descrizione dell'errore
    printf("Errore init SDL: %s\n", SDL_GetError());
    return 1;
  }

  // all'uscita del programma esegui SDL_Quit per risistemare le cose
  atexit(SDL_Quit);

  //dichiariamo un puntatore ad una superficie ed un int a 32 bit per i flag da assegnarle
  Uint32 flags = SDL_HWSURFACE|SDL_DOUBLEBUF;

  // per rendere disegnabile la Superficie bisogna utilizzare SDL_SetVideoMode
  if(!( screen = SDL_SetVideoMode(DIM_H, DIM_V, 0,flags) ))
  {
    printf("Problemi con il settaggio dello schermo: %s\n", SDL_GetError());
    return 1;
  }

  //nascondiamo il cursore della finestra
  SDL_ShowCursor(0);

  
  //carica img cursore su superficie temporanea
  if((temp = IMG_Load("mirino.png"))==NULL)
  {
    printf("Errore caricamento mirino: %s\n",IMG_GetError());
    return 1;
  }
  
  //Adatta l'img al display attuale e l'assegna ad una superficie
  cursor = SDL_DisplayFormat(temp);

  //si libera temp
  SDL_FreeSurface(temp);
  
  //settiamo trasparente il colore #000000 di cursor
  SDL_SetColorKey(cursor, SDL_SRCCOLORKEY|SDL_RLEACCEL, SDL_MapRGB(cursor->format, 0x0, 0x0, 0x0));

  // settiamo le flag per le nuove superfici da creare
  flags = SDL_HWSURFACE|SDL_SRCCOLORKEY;
  //creiamo una superficie uguale a cursor per coprire la scia lasciata dal movimento
  bg_cursor=SDL_CreateRGBSurface(flags,cursor->w,cursor->h,cursor->format->BitsPerPixel,0,0,0,0);
  //creiamo una superficie uguale a screen per salvare lo stato quando cambiamo modalit
  //di visualizzazione da finestra a tutto schermo
  screen_backup=SDL_CreateRGBSurface(flags,screen->w,screen->h,screen->format->BitsPerPixel,0,0,0,0);

  //variabile per il ciclo principale
  int done = 0;
  //contatore
  int i;
  
  //limiti della finestra in funzione delle dimensioni del cursore
  int x_limit = DIM_H-cursor->w;
  int y_limit = DIM_V-cursor->h;

  //definisco rettangolo delle dimensioni di cursor utilizzato per i Blit di superfici
  //e per memorizzare la posizione del cursore
  SDL_Rect rect_cur; 
  rect_cur.x = 0; 
  rect_cur.y = 0; 
  rect_cur.w = cursor->w; 
  rect_cur.h = cursor->h; 

  //due punti per le rette della griglia
  point p1,p2;
  //3 trutture RGB_Color inizializzate a rosso, giallo e bianco
  RGB_Color red = {0xFF,0x0,0x0};
  RGB_Color yellow = {0xFF,0xFF,0x0};
  RGB_Color white = {0xFF,0xFF,0xFF};

  //lato della cella
  int passo=50;
  
  //disegno righe orizzontali
  p1.x=0;
  p1.y=0;
  p2.x=DIM_H-1;
  p2.y=0;
  
  for(i=0;i<=DIM_V/passo;i++)
  {
    draw_line(screen,p1, p2, white);
    p1.y+=passo;
    p2.y+=passo;
  }
  
  //disegno ultima riga orizzontale
  p1.x=0;
  p1.y=DIM_V-1;
  p2.x=DIM_H-1;
  p2.y=DIM_V-1;
  draw_line(screen,p1, p2, white);

  //disegno righe verticali
  p1.x=0;
  p1.y=0;
  p2.x=0;
  p2.y=DIM_V-1;

  for(i=0;i<DIM_H/passo;i++)
  {
    draw_line(screen,p1, p2, white);
    p1.x+=passo;
    p2.x+=passo;
  }
  
  //disegno ultima riga verticale
  p1.x=DIM_H-1;
  p1.y=0;
  p2.x=DIM_H-1;
  p2.y=DIM_V-1;
  draw_line(screen,p1, p2, white);

  //vertice sx di un quadrato
  point cel;
  //indici di riga e di colonna della griglia
  int row, col, old_row=0,old_col=0;

  //dichiariamo una struttura SDL_Event
  SDL_Event event;

  //game loop, ovvero il loop che viene eseguito finch non si esce
  while(!done)  
  {
    // SDL_WaitEvent attende il prossimo evento
    SDL_WaitEvent(&event);
    
    //premendo la x della finestra col mouse si esce
    if( event.type == SDL_QUIT )  
      done = 1;  

    if( event.type == SDL_KEYDOWN )
    {
      //ma si esce anche premendo Esc
      if ( event.key.keysym.sym == SDLK_ESCAPE ) 
        done = 1; 

      //premendo spazio si passa dalla modalit schermo intero alla modalit 
      //finestra e viceversa
      if ( event.key.keysym.sym == SDLK_SPACE ) 
      {
        //salviamo l'immagine attuale in screen_backup
        SDL_BlitSurface(screen,NULL,screen_backup,NULL);
        //cambiamo modalit
        flags^=SDL_FULLSCREEN;
        screen = SDL_SetVideoMode(DIM_H, DIM_V, 0,flags);
        //ripristiniamo l'immagine 
        SDL_BlitSurface(screen_backup,NULL,screen,NULL);
      }
    }

    //premiamo il tasto sx del mouse, selezioniamo 1 punto
    if( event.button.button == BUTTON_SX && event.button.type == SDL_MOUSEBUTTONDOWN )
    {

      //calcoliamo l'indice di riga e di colonna della cella puntata
      row=(rect_cur.x+(cursor->w)/2)/passo;
      col=(rect_cur.y+(cursor->h)/2)/passo;

      //calcoliamo le cordinate del vertice superiore sx della cella
      cel.x=(row*passo)+(passo/2);
      cel.y=(col*passo)+(passo/2);

      //disegnamo il punto selezionato
      draw_cross(screen,cel,yellow);
    }
  
    //ci spostiamo con il mouse
    if( event.motion.type == SDL_MOUSEMOTION )
    {
      //setto le cordinate del rettangolo del puntatore   
      rect_cur.x=event.button.x;
      rect_cur.y=event.button.y;
      rect_cur.w = cursor->w; 
      rect_cur.h = cursor->h; 

      //se si vuole andare orizzontalmente oltre la finestra si viene bloccati
      if(rect_cur.x >= x_limit)
        rect_cur.x = x_limit;
      else if(rect_cur.x < 0)
        rect_cur.x = 0;
    
      //se si vuole andare verticalmente oltre la finestra si viene bloccati
      if(rect_cur.y >= y_limit)
        rect_cur.y = y_limit;
      else if(rect_cur.y < 0)
        rect_cur.y = 0;

      //calcoliamo la riga e la colonna su cui ci troviamo
      row=(rect_cur.x+(cursor->w)/2)/passo;
      col=(rect_cur.y+(cursor->h)/2)/passo;
      
      //se ci troviamo in una cella diversa aggiorniamo la selezione
      if(row!=old_row || col!=old_col)
      {
        //coloriamo la vecchia cella di bianco
        cel.x=old_row*passo;
        cel.y=old_col*passo;
        draw_square(screen,cel,passo,white);

        //coloriamo la nuova cella di rosso
        cel.x=row*passo;
        cel.y=col*passo;
        draw_square(screen,cel,passo,red);

        //aggiorniamo i vecchi indici di cella
        old_row=row;
        old_col=col;
      }
    }

    //recuperiamo la parte di screen prima di muoverci
    SDL_BlitSurface(screen, &rect_cur, bg_cursor, NULL); 
      
    //disegniamo il cursore alla nuova posizione
    SDL_BlitSurface(cursor, NULL, screen, &rect_cur);
    
    //aggiorniamo lo schermo
    SDL_Flip(screen);
    
    //disegniamo la parte di screen recuperata per coprire la scia
    SDL_BlitSurface(bg_cursor, NULL, screen, &rect_cur);
  }

  return 0;
}

//disegna un pixel sulla superficie desiderata alle cordinate (x,y) con colore R,G,B
void draw_pixel(SDL_Surface *surface, int x, int y, RGB_Color s_color)
{
  //restituisce la mappatura di un colore RGB in base ai bpp (alla definizione)
  Uint32 color = SDL_MapRGB(surface->format, s_color.R, s_color.G, s_color.B);

  //byte per pixel
  int bpp = surface->format->BytesPerPixel;

  //pixel della superficie da colorare
  Uint8 *pixel = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

  //verifichiamo il numero di byte usati per la nostra superficie
  switch (bpp)
  {
    case 1: // 1 byte =>  8-bpp
      *pixel = color;
      break;
    case 2: // 2 byte => 16-bpp
      *(Uint16 *)pixel = color;
      break;
    case 3: // 3 byte => 24-bpp
      if(SDL_BYTEORDER == SDL_BIG_ENDIAN) 
      {
        pixel[0] = (color >> 16) & 0xff;
        pixel[1] = (color >> 8) & 0xff;
        pixel[2] = color & 0xff;
      } 
      else 
      {
        pixel[0] = color & 0xff;
        pixel[1] = (color >> 8) & 0xff;
        pixel[2] = (color >> 16) & 0xff;
      }
      break;
    case 4: // 4 byte => 32-bpp
      *(Uint32 *)pixel = color;
      break;
  }
}

//disegna una croce di 5 pixel su di una superficie alle cordinate specificate 
//del colore desiderato
void draw_cross(SDL_Surface *surface, point p, RGB_Color c_color)
{
  //blocchiamo lo schermo se necessario
  if ( SDL_MUSTLOCK(surface) )
    SDL_LockSurface(surface);

  //si disegnano i pixel della croce
  //IMPORTANTE: non ci sono check sulla posizione!
  draw_pixel(surface,p.x,p.y,c_color);
  draw_pixel(surface,p.x+1,p.y,c_color);
  draw_pixel(surface,p.x-1,p.y,c_color);
  draw_pixel(surface,p.x,p.y+1,c_color);
  draw_pixel(surface,p.x,p.y-1,c_color);

  //sblocchiamo lo schermo se necessario
  if ( SDL_MUSTLOCK(surface) )
    SDL_UnlockSurface(surface);
}

//disegnamo una linea su di una superficie dal punto p0 al punto p1 di colore color
void draw_line(SDL_Surface *surface,point p0, point p1, RGB_Color color)
{
  int i;
  int comp;
  int d_a,d_b;
  int a,b;
  int inc;

  //blocchiamo lo schermo se necessario
  if ( SDL_MUSTLOCK(surface) )
    SDL_LockSurface(surface);

  //disegnamo un solo punto 
  if(p0.x==p1.x && p0.y==p1.y)
      draw_pixel(surface,p0.x,p0.y,color);
   //disegnamo orizzontalmente
  else if(abs(p1.x-p0.x)>=abs(p1.y-p0.y))
  {
    d_a=p1.x-p0.x;
    d_b=p1.y-p0.y;
    a=p0.x;
    b=p1.x;
    
    //da sx a dx 
    if(p0.x<p1.x)
      inc=1;
    //da dx a sx
    else
      inc=-1;

    //serve per disegnare negli estremi compresi
    b+=inc;

    for(i=a;i!=b;i=i+inc)
    {
      //calcoliamo la y
      comp=(d_b*(i-a))/(d_a)+p0.y;  
      //disegnamo il pixel alle cordinate attuali
      draw_pixel(surface,i,comp,color);
    }

  }
  //disegnamo verticalmente
  else
  {
    d_a=p1.y-p0.y;
    d_b=p1.x-p0.x;
    a=p0.y;
    b=p1.y;
    
    //dal basso all'alto
    if(p0.y<p1.y)
      inc=1;
    //dall'alto al basso
    else
      inc=-1;

    //serve per disegnare negli estremi compresi
    b+=inc;
    
    for(i=a;i!=b;i=i+inc)
    {
      //calcoliamo la x
      comp=(d_b*(i-a))/(d_a)+p0.x;  
      //disegnamo il pixel alle cordinate attuali
      draw_pixel(surface,comp,i,color);
    }
  }

  //sblocchiamo lo schermo se necessario
  if ( SDL_MUSTLOCK(surface) )
    SDL_UnlockSurface(surface);
}

//disegnamo un quadrato di lato lato a partire dal vertice superiore sinistro del colore color
void draw_square(SDL_Surface *surface,point p, int lato, RGB_Color color)
{
  point temp;

  //lato superiore
  if(p.x+lato<DIM_H)
    temp.x=p.x+lato;
  else
    temp.x=DIM_H-1;
  temp.y=p.y;
  draw_line(surface,p,temp,color);
  
  //lato sx
  temp.x=p.x;
  if(p.y+lato<DIM_V)
    temp.y=p.y+lato;
  else
    temp.y=DIM_V-1;
  draw_line(surface,p,temp,color);
  
  //lato inferiore
  if(p.y+lato<DIM_V)
    p.y+=lato;
  else
    p.y=DIM_V-1;
  if(p.x+lato<DIM_H)
    temp.x=p.x+lato;
  else
    temp.x=DIM_H-1;
  temp.y=p.y;
  draw_line(surface,p,temp,color);
  
  //lato dx
  if(p.x+lato<DIM_H)
    p.x+=lato;
  else
    p.x=DIM_H-1;
  temp.x=p.x;
  temp.y=p.y-lato;
  draw_line(surface,p,temp,color);
}
